@@ -1,9 +1,13 @@
 import time
 from pathlib import Path
+import fnmatch
 import numpy as np
 import re # for sesame filename extraction
 import pandas as pd
-from importlib import resources # py3.7+
+try:
+    from importlib import resources # py >= 3.7
+except ImportError: # py < 3.7
+    import pkg_resources
 #app
 from .progress_bar import * # context tqdm
 
@@ -91,6 +95,11 @@ Use cases and format:
         Only those probes that pass the p-value cutoff will be included.
     format = beta_csv:
         for reading processed.csv files from methylprep, and forcing it NOT to load from the pickled beta dataframe file, if present.
+    format = poobah_csv:
+        similar to beta_csv, this pulls poobah p-values for all probes out of all processed CSV files into one dataframe.
+        These p-values will include failed probes and probes that would be filterd by quality_mask. 'poobah' excludes these.
+    format = poobah:
+        reads the 'poobah_values.pkl' file and returns a dataframe of p-values. Note failed / poor-quality probes are replaced with NaN.
 
 .. note::
    Science on p-value cutoff:
@@ -112,7 +121,7 @@ Use cases and format:
    - v0.6.3: added 'no_filter' step that automatically removes probes that illumina, the manufacturer, claims are sketchy for certain Catalog IDs. (Disable this with `no_filter=True`)
     """
     #1a: validate inputs
-    formats = ('beta_value', 'm_value', 'meth', 'meth_df', 'noob_df', 'sesame', 'beta_csv')
+    formats = ('beta_value', 'm_value', 'meth', 'meth_df', 'noob_df', 'beta_csv', 'poobah_csv', 'poobah', 'sesame')
     if format not in formats:
         raise ValueError(f"Check the spelling of your format. Allowed: {formats}")
 
@@ -122,7 +131,11 @@ Use cases and format:
     poobah_parts = []
     #1c: import meta data
     PROBE_FILE = 'illumina_sketchy_probes_996.npy'
-    with resources.path(pkg_namespace, PROBE_FILE) as probe_filepath:
+    try:
+        with resources.path(pkg_namespace, PROBE_FILE) as probe_filepath:
+            sketchy_probes = np.load(probe_filepath)
+    except:
+        probe_filepath = pkg_resources.resource_filename(pkg_namespace, PROBE_FILE)
         sketchy_probes = np.load(probe_filepath)
 
     #2a: return some simpler data formats, before trying betas/m_values and processed_csvs.
@@ -151,9 +164,12 @@ Use cases and format:
                 unmeth_dfs.append( pd.read_pickle(part) )
         if meth_dfs != [] and unmeth_dfs != []:
             tqdm.pandas()
-            meth_dfs = pd.concat(meth_dfs, axis='columns', join='inner').progress_apply(lambda x: x)
-            unmeth_dfs = pd.concat(unmeth_dfs, axis='columns', join='inner').progress_apply(lambda x: x)
-            LOGGER.info(f"{meth_dfs.shape} {unmeth_dfs.shape}")
+            try:
+                meth_dfs = pd.concat(meth_dfs, axis='columns', join='inner').progress_apply(lambda x: x)
+                unmeth_dfs = pd.concat(unmeth_dfs, axis='columns', join='inner').progress_apply(lambda x: x)
+                LOGGER.info(f"{meth_dfs.shape} {unmeth_dfs.shape}")
+            except pd.errors.InvalidIndexError as e:
+                LOGGER.error(f"Your meth/unmeth data contains duplicate probes of the same name. Use pandas to load instead.")
             return meth_dfs, unmeth_dfs
 
     elif format == 'noob_df':
@@ -168,15 +184,18 @@ Use cases and format:
                 unmeth_dfs.append( pd.read_pickle(part) )
         if meth_dfs != [] and unmeth_dfs != []:
             tqdm.pandas()
-            meth_dfs = pd.concat(meth_dfs, axis='columns', join='inner').progress_apply(lambda x: x)
-            unmeth_dfs = pd.concat(unmeth_dfs, axis='columns', join='inner').progress_apply(lambda x: x)
-            LOGGER.info(f"{meth_dfs.shape}, {unmeth_dfs.shape}")
+            try:
+                meth_dfs = pd.concat(meth_dfs, axis='columns', join='inner').progress_apply(lambda x: x)
+                unmeth_dfs = pd.concat(unmeth_dfs, axis='columns', join='inner').progress_apply(lambda x: x)
+                LOGGER.info(f"{meth_dfs.shape}, {unmeth_dfs.shape}")
+            except pd.errors.InvalidIndexError as e:
+                LOGGER.error(f"Your noob_meth/noob_unmeth data contains duplicate probes of the same name. Use pandas to load instead.")
             return meth_dfs, unmeth_dfs
 
     #2b: determine number of file parts
     # bug: total_parts will match 'meth_values' for format='meth' instead of reading csvs.
     # looking for multiple pickled files (beta/m)
-    if format in ('beta_value', 'm_value'):
+    if format in ('beta_value', 'm_value', 'poobah'):
         # .pkl or .pkl.gz OK
         total_parts = list(Path(filepath).rglob(f'{file_stem}{format}*.pkl*'))
         # or specify one file by name instead of a folder.
@@ -209,9 +228,11 @@ Use cases and format:
                         columns = ['IlmnID', 'm_value']
                     elif format == 'meth':
                         columns = ['IlmnID', 'noob_meth', 'noob_unmeth']
+                    elif format == 'poobah_csv':
+                        columns = ['IlmnID','poobah_pval']
                     else:
                         raise ValueError(f"format {format} not supported. fix.")
-                    if no_poobah == False:
+                    if no_poobah == False and 'poobah_pval' not in columns:
                         columns.append('poobah_pval')
                     index_column = 'IlmnID' if 'IlmnID' in columns else columns[0] # fragile assumption: first column is index
                     sample = pd.read_csv(part,
@@ -231,6 +252,7 @@ Use cases and format:
                         # that will auto-set the index to 'illumina_id'
                         sample.rename_axis('IlmnID', inplace=True)
                 except ValueError as e:
+                    print(f"DEBUG - could not read csv parts; trying fallback method")
                     sample = pd.read_csv(part)
                     if 'IlmnID' in sample.columns:
                         sample.set_index('IlmnID', inplace=True)
@@ -281,6 +303,13 @@ Use cases and format:
                     sample_betas.append(col)
                     if verbose and not silent:
                         LOGGER.info(f'{sample_name}, {col.shape} --> {len(sample_betas)}')
+                elif format == 'poobah_csv' and 'poobah_pval' not in sample.columns:
+                    raise ValueError("Cannot read poobah_pval from processed CSVs; missing")
+                elif format == 'poobah_csv' and 'poobah_pval' in sample.columns:
+                    col = sample.loc[:, ['poobah_pval']]
+                    col.rename(columns={'poobah_pval': sample_name}, inplace=True)
+                    sample_names.append(sample_name)
+                    sample_betas.append(col)
                 elif (column_names is not None and
                     len(column_names) == 1 and
                     column_names[0] in sample.columns and
@@ -340,7 +369,11 @@ Use cases and format:
                 return data_containers
 
             LOGGER.info('merging...')
-            df = pd.concat(sample_betas, axis='columns', join='inner').progress_apply(lambda x: x)
+            try:
+                df = pd.concat(sample_betas, axis='columns', join='inner').progress_apply(lambda x: x)
+            except pd.errors.InvalidIndexError as e:
+                LOGGER.error(f"Merge Failed: ({e}) so returning each sample separately in a list")
+                return sample_betas
             ### HERE -- save the pieces of the DF and it merges 8X faster
             #npy = df.to_numpy()
             #parts.append(npy)
@@ -370,7 +403,7 @@ Use cases and format:
             probes_per_sample = int(sum([sub_df.shape[0] for sub_df in df.values()])/len(df.values()))
             if verbose: LOGGER.info(f"Reading a dictionary of dataframes with {len(df)} samples and {probes_per_sample} probes.")
             if verbose: LOGGER.info("No futher merging or poobah filtering applied to this file.")
-            return df
+            return df # e.g. beta_values, m_values, poobah_values from pickle
 
         # ensure probes are in rows.
         if df.shape[0] < df.shape[1]:
@@ -481,6 +514,8 @@ Arguments:
             LOGGER.warning(f'Columns in sample sheet meta data files does not match for these files and cannot be combined:'
                            f'{[str(i) for i in meta_files]}')
             meta = pd.read_pickle(meta_files[0])
+            if any(meta.columns.duplicated()):
+                meta = meta.loc[:, ~meta.columns.duplicated()]
             partial_meta = True
         else:
             meta = pd.concat(frames, axis=0, sort=False)
@@ -488,6 +523,8 @@ Arguments:
 
     if len(meta_files) == 1:
         meta = pd.read_pickle(meta_files[0])
+        if any(meta.columns.duplicated()):
+            meta = meta.loc[:, ~meta.columns.duplicated()]
     elif multiple_metas:
         if partial_meta:
             LOGGER.info("Multiple meta_data found. Only loading the first file.")
@@ -555,6 +592,10 @@ Arguments:
         data_df = data_df.rename(columns=sample_mapping)
         LOGGER.info("Renamed data_df.index and meta_df.columns to Sample_Name")
 
+    # ensure data_df returns probes in rows
+    if data_df.shape[0] < data_df.shape[1]:
+        data_df = data_df.transpose()
+
     return data_df, meta
 
 
@@ -793,3 +834,58 @@ def load_sesame(filepath='.',
         return df
     else:
         LOGGER.info("No sesame files found.")
+
+
+'''
+def load_all_betas(path):
+    """ loads methylprep processed data and returns a single dataframe of all non-control probe betas, similar
+    to sesame 3.13+ standard export format. Path should contain all of the pickled output files to merge.
+    (combines CpG probes, SNPs, and mouse-experimental probes in on dataframe)
+
+    THIS IS NOT CALLED ANYWHERE"""
+    extra_files_found = []
+    patterns = {
+        'beta': '*beta_values*.pkl',
+        'con': '*control_probes*.pkl',
+        'mouse': '*mouse_probes*.pkl',
+    }
+    foundb = [str(file.name) for file in Path(path).rglob(patterns['beta'])]
+    if foundb:
+        cg = Path(path, foundb[0]).expanduser()
+        df1 = pd.read_pickle(cg)
+    if len(foundb) > 1:
+        extra_files_found.append(foundb[1:])
+
+    foundc = [str(file.name) for file in Path(path).rglob(patterns['con'])]
+    if foundc:
+        con = Path(path, foundc[0]).expanduser()
+        dfcon = pd.read_pickle(con) # a list of dicts
+    if len(foundc) > 1:
+        extra_files_found.append(foundc[1:])
+
+    foundm = [str(file.name) for file in Path(path).rglob(patterns['mouse'])]
+    if foundm:
+        mouse = Path(path, foundm[0]).expanduser()
+        if mouse.exists() and mouse.is_file():
+            ms = pd.read_pickle(mouse)
+    else:
+        ms = None
+    if len(foundm) > 1:
+        extra_files_found.append(foundm[1:])
+
+    samples = list(df1.columns)
+    col_data = []
+    for sample in samples:
+        df2 = dfcon[sample].loc[ ~dfcon[sample]['snp_beta'].isna() ][['snp_beta']]
+        df2 = df2.rename(columns={'snp_beta':sample})
+        if isinstance(ms, dict):
+            df3 = ms[sample][['beta_value']]
+            df3 = df3.rename(columns={'beta_value':sample})
+        else:
+            df3 = pd.DataFrame()
+        one_col = pd.concat([df1[[sample]], df2, df3])
+        col_data.append(one_col)
+    if extra_files_found:
+        LOGGER.info("Found extra files and ignored them: {extra_files_found}")
+    return pd.concat(col_data, axis='columns')
+'''
