@@ -38,7 +38,7 @@ inputs:
         if not specified, it will load the data from data_source and determine the array for you.
     median_cutoff
         the minimum difference in the medians of X and Y probe copy numbers to assign male or female
-        (copied from the minif sex predict function)
+        (copied from the minfi sex predict function)
     include_probe_failure_percent:
         True: includes poobah percent per sample as column in the output table and on the plot.
         Note: you must supply a 'path' as data_source to include poobah in plots.
@@ -55,7 +55,7 @@ inputs:
     return_labels: (requires plot == True)
         When using poobah_cutoff, the figure only includes A-Z,1...N labels on samples on plot to make it easier to read.
         So to get what sample_ids these labels correspond to, you can rerun the function with return_labels=True and it will
-        skip plotting and just return a dataframe with sample_ids and these labels, to embed in a PDF report if you like.
+        skip plotting and just return a dictionary with sample_ids and these labels, to embed in a PDF report if you like.
     custom_label:
         Option to provide a dictionary with keys as sample_ids and values as labels to apply to samples.
         e.g. add more data about samples to the multi-dimensional QC plot
@@ -92,8 +92,8 @@ Note: ~90% of Y probes should fail if the sample is female. That chromosome is m
             meth, unmeth = methylcheck.qc_plot._get_data(
                 data_containers=None, path=data_source,
                 compare=False, noob=True, verbose=False)
-        if include_probe_failure_percent == True and Path(data_source,'poobah_values.pkl').exists():
-            poobah = pd.read_pickle(Path(data_source,'poobah_values.pkl'))
+        if include_probe_failure_percent == True and Path(data_source,'poobah_values.pkl').expanduser().exists():
+            poobah = pd.read_pickle(Path(data_source,'poobah_values.pkl').expanduser())
 
     elif data_source_type in ('container'):
         # this will look for saved pickles first, then csvs or parsing the containers (which are both slower)
@@ -102,7 +102,7 @@ Note: ~90% of Y probes should fail if the sample is female. That chromosome is m
             data_containers=data_source, path=None,
             compare=False, noob=False, verbose=False)
 
-    elif data_source_type is 'meth_unmeth_tuple':
+    elif data_source_type == 'meth_unmeth_tuple':
         (meth, unmeth) = data_source
 
     if len(meth) != len(unmeth):
@@ -120,7 +120,7 @@ Note: ~90% of Y probes should fail if the sample is female. That chromosome is m
     if verbose:
         LOGGER.debug(array_type)
     LOGGER.setLevel(logging.WARNING)
-    manifest = Manifest(array_type, on_lambda=on_lambda)._Manifest__data_frame # 'custom', '27k', '450k', 'epic', 'epic+'
+    manifest = Manifest(array_type, on_lambda=on_lambda, verbose=verbose)._Manifest__data_frame # 'custom', '27k', '450k', 'epic', 'epic+'
     LOGGER.setLevel(logging.INFO)
     x_probes = manifest.index[manifest['CHR']=='X']
     y_probes = manifest.index[manifest['CHR']=='Y']
@@ -149,6 +149,7 @@ Note: ~90% of Y probes should fail if the sample is female. That chromosome is m
 
     # median cutoff - can be manipulated by user --- default = -2 --- used to predict sex
     sex0 = ['F' if x < median_cutoff else 'M' for x in median_difference]
+    # NOTE for testing: GSE85566/GPL13534 (N=120) has 4 samples that are predicted as wrong sex when using -2, but work at -0.5.
 
     # populate dataframe with predicted sex
     output['predicted_sex'] = sex0
@@ -225,7 +226,7 @@ data columns: ['x_median', 'y_median', 'predicted_sex', 'X_fail_percent', 'Y_fai
 - omits labels for samples that have LOW failure rates, but shows IDs when failed
 - adds legend of sketchy samples and labels
 - show delta age on labels (using custom column dict)
-- unit tests with custom label and without, and check that beadarray report still works with this function
+- unit tests with custom label and without, and check that controls_report still works with this function
 - save_fig
 - return_labels, returns a lookup dict instead of plot
 
@@ -237,7 +238,6 @@ Dicts must match the data DF index.
     else:
         LOGGER.warning("sample_failure_percent index did not align with output data index")
     #sns.set_theme(style="white")
-    custom_palette = sns.set_palette(sns.color_palette(['#FE6E89','#0671B7']))
     show_mismatches = None if 'sex_matches' not in data.columns else "sex_matches"
     if show_mismatches:
         data["sex_matches"] = data["sex_matches"].map({0:"Mismatch", 1:"Match"})
@@ -248,6 +248,14 @@ Dicts must match the data DF index.
         if poobah_range < poobah_cutoff/2:
             show_failure = None
             sample_sizes = (40,40)
+
+    custom_palette = sns.set_palette(sns.color_palette(['#FE6E89','#0671B7']))
+    # if only one sex, make sure male is blue; female is pink
+    # if hasattr(output, 'actual_sex') and set(output.actual_sex) == set('M')
+    # if first value to be plotted is male, change palette
+    if hasattr(data, 'predicted_sex') and list(data.predicted_sex)[0] == 'M':
+        custom_palette = sns.set_palette(sns.color_palette(['#0671B7','#FE6E89']))
+
     fig = sns.relplot(data=data,
         x='x_median',
         y='y_median',
@@ -286,7 +294,7 @@ Dicts must match the data DF index.
     else:
         ax.set_title(f"Predicted sex based on matching X and Y probes.")
     if save:
-        filepath = 'predicted_sexes.png' if data_source_type != 'path' else Path(data_source,'predicted_sexes.png')
+        filepath = 'predicted_sexes.png' if data_source_type != 'path' else Path(data_source,'predicted_sexes.png').expanduser()
         plt.savefig(filepath, bbox_inches="tight")
     if return_fig:
         return fig
@@ -294,7 +302,7 @@ Dicts must match the data DF index.
 
 def _fetch_actual_sex_from_sample_sheet_meta_data(filepath, output):
     """output is a dataframe with Sample_ID in the index. This adds actual_sex as a column and returns it."""
-    # bead array does the same thing, and only calls get_sex() with the minimum of data to be fast, because these are already loaded. Just passes in meth/unmeth data
+    # controls_report() does the same thing, and only calls get_sex() with the minimum of data to be fast, because these are already loaded. Just passes in meth/unmeth data
     # Sample sheet should have 'M' or 'F' in column to match predicted sex.
 
     # merge actual sex into processed output, if available
@@ -306,14 +314,25 @@ def _fetch_actual_sex_from_sample_sheet_meta_data(filepath, output):
     }
     loaded_files = {}
     for file_pattern in file_patterns:
-        for filename in Path(filepath).rglob(file_pattern):
+        for filename in Path(filepath).expanduser().rglob(file_pattern):
             if '.pkl' in filename.suffixes:
                 loaded_files['meta'] = pd.read_pickle(filename)
                 break
             if '.csv' in filename.suffixes:
-                loaded_files['meta'] = pd.read_csv(filename)
+                try:
+                    from methylprep.files import SampleSheet
+                except ImportError:
+                    raise ImportError("parsing a sample sheet CSV requires `methylprep` be installed first.")
+                #uses methylprep.files.SampleSheet() instead of --- loaded_files['meta'] = pd.read_csv(filename) --- to support legacy [header] format(s).
+                sample_sheet = SampleSheet(filename, filepath)
+                loaded_files['meta'] = sample_sheet._SampleSheet__data_frame
                 break
     if len(loaded_files) == 1:
+        # methylprep v1.5.4-6 was creating meta_data files with two Sample_ID columns. Check and fix here:
+        # methylcheck 0.7.9 / prep 1.6.0 meta_data lacking Sample_ID when sample_sheet uses alt column names and gets replaced.
+        if any(loaded_files['meta'].columns.duplicated()):
+            loaded_files['meta'] = loaded_files['meta'].loc[:, ~loaded_files['meta'].columns.duplicated()]
+            LOGGER.info("Removed a duplicate Sample_ID column in samplesheet")
         if 'Sample_ID' in loaded_files['meta'].columns:
             loaded_files['meta'] = loaded_files['meta'].set_index('Sample_ID')
         elif 'Sentrix_ID' in loaded_files['meta'].columns and 'Sentrix_Position' in loaded_files['meta'].columns:
@@ -336,26 +355,36 @@ def _fetch_actual_sex_from_sample_sheet_meta_data(filepath, output):
             elif 'Sex' in renamed_columns.values():
                 renamed_column = 'Sex'
         if renamed_column is not None:
-            # next, ensure samplesheet Sex/Gender (Male/Female) are recoded as M/F; beadarray does NOT do this step, but should.
+            # next, ensure samplesheet Sex/Gender (Male/Female) are recoded as M/F; controls_report() does NOT do this step, but should.
             sex_values = set(loaded_files['meta'][renamed_column].unique())
+            #print('sex_values', sex_values)
             if not sex_values.issubset(set(['M','F'])): # subset, because samples might only contain one sex
-                if 'Male' in sex_values and 'Female' in sex_values:
+                if 'Male' in sex_values or 'Female' in sex_values:
                     loaded_files['meta'][renamed_column] = loaded_files['meta'][renamed_column].map({'Male':'M', 'Female':'F'})
-                elif 'male' in sex_values and 'female' in sex_values:
+                elif 'male' in sex_values or 'female' in sex_values:
                     loaded_files['meta'][renamed_column] = loaded_files['meta'][renamed_column].map({'male':'M', 'female':'F'})
-                elif 'MALE' in sex_values and 'FEMALE' in sex_values:
+                elif 'MALE' in sex_values or 'FEMALE' in sex_values:
                     loaded_files['meta'][renamed_column] = loaded_files['meta'][renamed_column].map({'MALE':'M', 'FEMALE':'F'})
+                elif 'm' in sex_values or 'f' in sex_values:
+                    loaded_files['meta'][renamed_column] = loaded_files['meta'][renamed_column].map({'m':'M', 'f':'F'})
                 else:
-                    raise ValueError(f"Cannot compare with predicted sex because actual sexes listed in your samplesheet are not M or F: (found {sex_values})")
-
+                    raise ValueError(f"Cannot compare with predicted sex because actual sexes listed in your samplesheet are not understood (expecting M or F): (found {sex_values})")
             output['actual_sex'] = None
             output['sex_matches'] = None
             for row in output.itertuples():
-                actual_sex = loaded_files['meta'].loc[row.Index].get(renamed_column)
+                try:
+                    actual_sex = str(loaded_files['meta'].loc[row.Index].get(renamed_column))
+                except KeyError:
+                    if 'Sample_ID' in output.columns:
+                        LOGGER.warning("Sample_ID was another column in your output DataFrame; Set that to the index when you pass it in.")
+                    raise KeyError("Could not read actual sex from meta data to compare.")
                 if isinstance(actual_sex, pd.Series):
                     LOGGER.warning(f"Multiple samples matched actual sex for {row.Index}, because Sample_ID repeats in sample sheets. Only using first match, so matches may not be accurate.")
                     actual_sex = actual_sex[0]
-                sex_matches = 1 if actual_sex.upper() == row.predicted_sex.upper() else 0
+                if hasattr(row,'predicted_sex'):
+                    sex_matches = 1 if actual_sex.upper() == str(row.predicted_sex).upper() else 0
+                else:
+                    sex_matches = np.nan
                 output.loc[row.Index, 'actual_sex'] = actual_sex
                 output.loc[row.Index, 'sex_matches'] = sex_matches
         else:
