@@ -2,7 +2,7 @@
 from pathlib import Path
 from io import StringIO
 import logging
-
+import fnmatch
 # calcs
 import warnings
 import math
@@ -16,18 +16,14 @@ import methylcheck
 
 LOGGER = logging.getLogger(__name__)
 
-__all__ = ['BeadArrayControlsReporter', 'controls_report']
+__all__ = ['ControlsReporter', 'controls_report']
 
 
-class BeadArrayControlsReporter():
-    """Class used by controls_report() to produce XLSX file similar to Illumina's Bead Array Controls Report software.
-
-    This will load all the methylprep control and raw output data, then perform all the calculations listed in
-    https://support.illumina.com/content/dam/illumina-support/documents/documentation/chemistry_documentation/infinium_assays/infinium_hd_methylation/beadarray-controls-reporter-user-guide-1000000004009-00.pdf
-    and then produce a tidy color-coded XLSX file with results.
-
-    This function is analogous to methylcheck.plot_controls() except that the output if a color-coded excel sheet instead of charts.
+class ControlsReporter():
+    """Class used by controls_report() to produce XLSX summary of control probe performance.
 
+    This will load all the methylprep control and raw output data, then perform the calculations recommended by manufacturer
+    then produce a tidy color-coded XLSX file with results. This function is analogous to methylcheck.plot_controls() except that the output if a color-coded excel sheet instead of charts.
     Last column "Result" will include OK (green), MARGINAL (yellow), or FAIL (RED) -- as a summary of all other tests.
 
     If there is a meta data pickle, and there's a Sex or Gender column, it will compare SEX.
@@ -39,12 +35,15 @@ class BeadArrayControlsReporter():
     input_filenames = {
         'control_probes.pkl': 'control',
         'poobah_values.pkl': 'poobah',
-        'sample_sheet_meta_data.pkl': 'samplesheet',
         # FUTURE?? load these separately, and only if there is a reason to run a sex prediction. But sex done by default.
-        # sex_prediction_files = {
         'noob_meth_values.pkl': 'noob_meth',
         'noob_unmeth_values.pkl': 'noob_unmeth',
     }
+    samplesheet_patterns = {
+        '*meta_data*.pkl': 'samplesheet', # meta_data is used first, if both available
+        '*samplesheet*.csv': 'samplesheet',
+        '*sample_sheet*.csv': 'samplesheet',
+    }
     # does NOT use m_values or 'beta_values.pkl'
 
     legacy_columns = {
@@ -99,8 +98,8 @@ class BeadArrayControlsReporter():
         'Predicted Sex']
 
     def __init__(self, filepath, outfilepath=None, bg_offset=3000, cutoff_adjust=1.0, colorblind=False,
-        roundoff=2, legacy=False, pval=True, pval_sig=0.05, passing=0.7):
-        self.filepath = filepath
+        roundoff=2, legacy=False, pval=True, pval_sig=0.05, passing=0.7, project_name=None):
+        self.filepath = filepath # folder with methyprep processed data
         self.bg_offset = bg_offset
         self.cut = cutoff_adjust # for adjusting minimum passing per test
         self.legacy = legacy # how output XLSX should be formatted
@@ -108,21 +107,42 @@ class BeadArrayControlsReporter():
         self.pval = pval # whether to include poobah in tests
         self.pval_sig = pval_sig # significance level to define a failed probe
         self.passing = passing # fraction of tests that all need to pass for sample to pass
+        self.project_name = project_name # used to name the QC report, if defined.
         # if outfilepath is not provided, saves to the same folder where the pickled dataframes are located.
         if not outfilepath:
             self.outfilepath = filepath
         else:
             self.outfilepath = outfilepath
+
         for filename in Path(filepath).rglob('*.pkl'):
             if filename.name in self.input_filenames.keys():
-                #vars()[self.input_filenames[filename.name]] = pd.read_pickle(filename)
                 setattr(self, self.input_filenames[filename.name], pd.read_pickle(filename))
+        # fuzzy matching samplesheet
+        for filename in Path(filepath).rglob('*'):
+            if any([fnmatch.fnmatch(filename.name, samplesheet_pattern) for samplesheet_pattern in self.samplesheet_patterns.keys()]):
+                #label = next(label for (patt,label) in samplesheet_patterns.items() if fnmatch.fnmatch(filename.name, patt))
+                if '.pkl' in filename.suffixes:
+                    setattr(self, 'samplesheet', pd.read_pickle(filename))
+                elif '.csv' in filename.suffixes:
+                    try:
+                        from methylprep.files import SampleSheet
+                    except ImportError:
+                        raise ImportError("parsing a sample sheet CSV requires `methylprep` be installed first.")
+                    #uses methylprep.files.SampleSheet() instead of --- pd.read_csv(filename) --- to support legacy [header] format(s).
+                    sample_sheet = SampleSheet(filename, filepath)
+                    setattr(self, 'samplesheet', sample_sheet._SampleSheet__data_frame)
+                break
+
         if not hasattr(self,'control'):
             raise FileNotFoundError(f"Could not locate control_probes.pkl file in {filepath}")
         if not hasattr(self,'poobah') and self.pval is True:
             raise FileNotFoundError(f"Could not locate poobah_values.pkl file in {filepath}; re-run and set 'pval=False' to skip calculating probe failures.")
         if hasattr(self,'samplesheet'):
             if isinstance(self.samplesheet, pd.DataFrame):
+                # methylprep v1.5.4-6 was creating meta_data files with two Sample_ID columns. Check and fix here:
+                if any(self.samplesheet.columns.duplicated()):
+                    self.samplesheet = self.samplesheet.loc[:, ~self.samplesheet.columns.duplicated()]
+                    LOGGER.info("Removed a duplicate Sample_ID column in samplesheet")
                 if 'Sample_ID' in self.samplesheet:
                     self.samplesheet = self.samplesheet.set_index('Sample_ID')
                 elif 'Sentrix_ID' in self.samplesheet and 'Sentrix_Position' in self.samplesheet:
@@ -158,7 +178,6 @@ class BeadArrayControlsReporter():
             self.cmid =  '#FFDD71'
             self.cfail = '#69B764'
 
-
     def process_sample(self, sample, con):
         """ process() will run this throug all samples, since structure of control data is a dict of DFs
         bg_offset = Background correction offset.
@@ -166,10 +185,8 @@ class BeadArrayControlsReporter():
             (applies to all background calculations, indicated with (bkg +x).)
 
 NEGATIVE control probes are used as the baseline for p-val calculations.
-    (but NEGATIVE and NORM_ not used in BeadArray)
 
-see also https://support.illumina.com/content/dam/illumina-support/documents/documentation/chemistry_documentation/infinium_assays/infinium_hd_methylation/infinium-hd-methylation-guide-15019519-01.pdf
-for list of expected intensity per type
+see also infinium-hd-methylation-guide-15019519-01.pdf for list of expected intensity per type
 
 MOUSE conversions (or proxy)
 baseline_G Extension -- missing -- use NEGATIVE Hairpin probes as proxy
@@ -271,7 +288,7 @@ target removal
         else:
             # BS min(C1, 2, or 3) / BS max(U1, 2, 3) > 1
             # META NOTE: BS Conversion I-C1 is "I C1" in 450k. U1 also differs.
-            # META NOTE: in order to match BeadArray, I had to ignore I-C3 and I-U3, as these always gave really low intensity outputs. The C1/U2 combo seems to work in practice.
+            # META NOTE: I had to ignore I-C3 and I-U3, as these always gave really low intensity outputs. The C1/U2 combo seems to work in practice.
             self.bisulfite_conversion_I_green_CU = round( min([
                 con[con['Extended_Type'].isin(['BS Conversion I-C1','BS Conversion I C1'])]['Mean_Value_Green'].values[0],
                 con[con['Extended_Type']=='BS Conversion I-C2']['Mean_Value_Green'].values[0],
@@ -308,7 +325,7 @@ target removal
                 con[con['Extended_Type']=='BS Conversion I-U5']['Mean_Value_Red'].values[0]
                 ]), self.roundoff)
 
-            #### min & max derived by comparing with BeadArray output, because guide PDF was unclear.
+            #### min & max derived by comparing with expected output, because guide manufacturer's reference guide was unclear here.
             # Bisulfite Conversion II min(Red) > C max(Green)
             self.bisulfite_conversion_II_red_ratio = round( min([
                 con[con['Extended_Type']=='BS Conversion II-1']['Mean_Value_Red'].values[0],
@@ -360,7 +377,7 @@ target removal
                 con[con['Extended_Type']=='Non_Specific_II_1_1']['Mean_Value_Green'].values[0],
                 ]), self.roundoff)
         else:
-            # ignoring controls 4,5,6 gave me the same output as BeadArray
+            # ignoring controls 4,5,6 gave me the expected output
             # Specificity I Green (min(PM)/max(MM)) > 1
             self.specificity_I_green = round( min([
                 con[con['Extended_Type']=='GT Mismatch 1 (PM)']['Mean_Value_Green'].values[0],
@@ -378,7 +395,7 @@ target removal
                 #con[con['Extended_Type']=='GT Mismatch 6 (MM)']['Mean_Value_Green'].values[0],
                 ]), self.roundoff)
 
-            # ignoring controls 1,2,3 here gave me the same output as BeadArray
+            # ignoring controls 1,2,3 here gave me the expected result
             # Specificity I Red (min(PM)/max(MM)) > 1
             self.specificity_I_red = round( min([
                 #con[con['Extended_Type']=='GT Mismatch 1 (PM)']['Mean_Value_Red'].values[0],
@@ -447,7 +464,7 @@ target removal
                 con[con['Extended_Type']=='NP (G)']['Mean_Value_Red'].values[0],
                 ]), self.roundoff)
 
-        # ADDITIONAL tests (not in BeadArray)
+        # ADDITIONAL tests
         self.negative_control_mean_green = round( np.mean(
         con[con['Control_Type'] == 'NEGATIVE']['Mean_Value_Green'].values,
         ))
@@ -583,8 +600,8 @@ target removal
             {'col': 'Baseline Red', 'val':baseline_R - self.bg_offset, 'formula': "max(Extension (C), Extension (G)) no offset", 'max':800, 'med':400, 'min':100},
             {'col': 'Negative Baseline G', 'val':self.negative_control_mean_green, 'formula': "mean NEGATIVE Green control probes"},
             {'col': 'Negative Baseline R', 'val':self.negative_control_mean_red, 'formula': "mean NEGATIVE Red control probes"},
-            {'col': 'Regression NORM_GA', 'val':self.regression_NORM_GA, 'formula': "NORM_G (grn) vs NORM_A (red)", 'max':0.9, 'med':0.9, 'min':0.9},
-            {'col': 'Regression NORM_CT', 'val':self.regression_NORM_CT, 'formula': "NORM_C (grn) vs NORM_T (red)", 'max':0.9, 'med':0.9, 'min':0.9},
+            {'col': 'Regression NORM_GA', 'val':self.regression_NORM_GA, 'formula': "NORM_G (grn) vs NORM_A (red)", 'max':0.8, 'med':0.8, 'min':0.8},
+            {'col': 'Regression NORM_CT', 'val':self.regression_NORM_CT, 'formula': "NORM_C (grn) vs NORM_T (red)", 'max':0.8, 'med':0.8, 'min':0.8},
             {'col': 'GCT score', 'val':self.gct_score, 'formula': "mean(oobG extC)/mean(oobG extT)", 'max':1.0, 'med':0.99, 'min':0.93},
             #{'col': 'NORM_A', 'val':self.norm_A_mean_green, 'formula': "mean NORM_A control probes Green)", 'max':600, 'med':300, 'min':100},
             #{'col': 'NORM_T', 'val':self.norm_T_mean_green, 'formula': "mean NORM_T control probes Green)", 'max':400, 'med':200, 'min':100},
@@ -608,6 +625,9 @@ target removal
                 # and mouse is not supported...
 
         row.update({k['col']:k['val'] for k in self.data[sample]})
+        # DEBUG: everything is rounding OKAY at this point
+        #if any([ (len(str(v).split(".")[1]) if '.' in str(v) else 0) > self.roundoff for k,v in row.items()]):
+        #    print('ERROR', [len(str(v).split(".")[1]) if '.' in str(v) else 0 for k,v in row.items()] )
         self.report.append(row) # this is converted to excel sheet, but self.data is used to format the data in save()
         self.formulas.update( {k['col']:k['formula'] for k in list(self.data.values())[0]} )
         # process() adds predicted sex and result column
@@ -622,17 +642,14 @@ target removal
         for sample,con in self.control.items():
             self.process_sample(sample, con) # saves everything on top of last sample, for now. testing.
 
-        #print(self.gct_scores)
-        #import pdb;pdb.set_trace()
-
         # predicted_sex, x_median, y_median, x_fail_percent, y_fail_percent,
-        if isinstance(self.samplesheet, pd.DataFrame) and hasattr(self, 'noob_meth') and hasattr(self, 'noob_unmeth'):
+        if hasattr(self, 'samplesheet') and isinstance(self.samplesheet, pd.DataFrame) and hasattr(self, 'noob_meth') and hasattr(self, 'noob_unmeth'):
             if self.predict_sex:
                 LOGGER.info("Predicting Sex and comparing with sample meta data...")
             else:
                 LOGGER.info("Predicting Sex...")
             try:
-                print(self.noob_meth.shape, self.noob_unmeth.shape)
+                #print(self.noob_meth.shape, self.noob_unmeth.shape)
                 sex_df = methylcheck.get_sex((self.noob_meth, self.noob_unmeth), array_type=None, verbose=False, plot=False, on_lambda=False, median_cutoff= -2, include_probe_failure_percent=False)
             except ValueError as e:
                 if str(e).startswith('Unsupported Illumina array type'):
@@ -642,25 +659,26 @@ target removal
                 else:
                     LOGGER.warning(e)
                     sex_df = pd.DataFrame()
-            # merge into processed output (self.report and self.data)
+            # merge into processed output (self.report and self.data); merge CORRECTED actual sex values, as M or F
+            sex_df = methylcheck.predict.sex._fetch_actual_sex_from_sample_sheet_meta_data(self.filepath, sex_df)
+
             # row index values are sample names
             for row in sex_df.itertuples():
                 try:
                     item_order = [idx for idx,item in enumerate(self.report) if item['Sample'] == row.Index][0]
                 except IndexError:
                     # in tests, this happens if samplesheet has data that isn't in process results
-                    print("report and sex_df not matching")
-                    import pdb;pdb.set_trace()
+                    raise IndexError("651: Report and sex_df not matching. This can happen if the samplesheet contains samples that aren't in processed results.")
 
                 self.data[row.Index].append({'col': 'Predicted Sex', 'val': row.predicted_sex, 'formula': f"X/Y probes"})
                 self.report[item_order]['Predicted Sex'] = row.predicted_sex
                 if self.predict_sex:
-                    # implies sample sheet exists and has Sex or Gender column; case WILL match now, because fixed when loaded.
-                    actual_sex = (self.samplesheet.loc[row.Index].get('Sex') or self.samplesheet.loc[row.Index].get('Gender'))
-                    sex_match = 1 if actual_sex.upper() == row.predicted_sex.upper() else 0
-                    self.data[row.Index].append({'col': 'Sex Match', 'val': sex_match,
-                        'formula': f"Sample sheet M/F", 'min':0, 'med':0.5, 'max':1.0})
-                    self.report[item_order]['Sex Match'] = sex_match
+                    self.data[row.Index].append({
+                        'col': 'Sex Match',
+                        'val': row.sex_matches,
+                        'formula': f"Sample sheet M/F",
+                        'min':0, 'med':0.5, 'max':1.0})
+                    self.report[item_order]['Sex Match'] = row.sex_matches
                     self.formulas['Sex Match'] = "Sample sheet M/F"
                 self.formulas['Predicted Sex'] = "X/Y probes"
                 # next attrib is not used in XLSX report, but saved to object in case others want to extend this.
@@ -682,22 +700,28 @@ target removal
         for col in self.data[sample]:
             if col['col'] in self.untestable_columns:
                 continue
+            if pd.isna(col['val']):
+                continue
             if col.get('max') is None:
                 print(f"WARNING: no threshold for {col['col']}")
                 continue
-            if pd.isna(col['val']):
-                continue
             passfail[col['col']] = 1 if col['val'] >= col['max'] else 0
-            missed_by_much[col['col']] = round(col['val'] / (col['max'] + 0.00000000001),2)
+            missed_by_amount = round(col['val'] / (col['max'] + 0.00000000001),2)
+            if missed_by_amount <= 0: # some tests can go negative, so setting a floor
+                # print(f"missed_by_amount: {missed_by_amount} col val {col['val']} col max {col['max']}")
+                missed_by_amount = 0.01
+            missed_by_much[col['col']] = missed_by_amount
 
 
         result = round( sum(passfail.values()) / sum(~np.isnan(list(passfail.values()))), 2) # fraction passing
         # average ratio of value to cutoff per failing test
+        failed_tests = [col for col,val in missed_by_much.items() if passfail[col] == 0]
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", category=RuntimeWarning)
             how_close = round(np.nanmean(np.clip([val for col,val in missed_by_much.items() if passfail[col] == 0], 0, 100000)),2)
 
-        if result == 1.0:
+        # percent of columns that passed
+        if result >= 1.0:
             result = 'OK'
         elif result >= self.passing and (how_close == 0 or np.isnan(how_close)):
             result = 'MARGINAL (infinite values)'
@@ -716,6 +740,8 @@ target removal
         self.data[sample].append(result_col)
         report_row = min([idx for idx,row in enumerate(self.report) if row['Sample'] == sample])
         self.report[report_row]['Result'] = result_col['val']
+        if failed_tests != []:
+            self.report[report_row]['Why Failed'] = ', '.join(failed_tests)
 
     def save(self):
         if self.legacy:
@@ -761,9 +787,11 @@ target removal
 
 
         # fetch the folder parent name and append to file (usually this is the project name)
-        project_name = Path(self.filepath).resolve().name
-        writer = pd.ExcelWriter(Path(self.outfilepath, f"{project_name}_QC_Report.xlsx"), engine='xlsxwriter')
-        report_df= pd.DataFrame(self.report)
+        if self.project_name == None:
+            self.project_name = Path(self.filepath).resolve().name
+        writer = pd.ExcelWriter(Path(self.outfilepath, f"{self.project_name}_QC_Report.xlsx"), engine='xlsxwriter')
+        report_df= pd.DataFrame(self.report) #<---- here in pandas 1.3x the decimal places get messed up.
+        report_df = report_df.round(self.roundoff)
         report_df.to_excel(writer, sheet_name='QC_REPORT', startcol=0, startrow=(2 if not self.legacy else 1), header=False, index=False)
         # Get the xlsxwriter objects from the dataframe writer object.
         workbook  = writer.book
@@ -813,7 +841,7 @@ target removal
                 worksheet.conditional_format(this_box, coloring)
                 # next: track if any values are longer than the column names, to adjust widths at end
                 if len(str(k['val'])) > col_widths[k['col']]:
-                    col_widths[k['col']] = len(str(k['val']))
+                    col_widths[k['col']] = int( len(str(k['val'])) * 1.5 )
 
         #worksheet.conditional_format('B2:B8', {'type': '3_color_scale', 'min_value':0, 'max_value':1, 'mid_value':0.5})
         # There is no way to specify “AutoFit” for a column in the Excel file format.
@@ -835,18 +863,16 @@ target removal
         writer.save()
 
 
-def controls_report(**kwargs):
-    """Run a clone of Illumina's Bead Array Controls Reporter to generate an excel QC Report.
+def controls_report(*args, **kwargs):
+    """Generates a color-coded excel/csv QC Report.
 
 CLI: methylcheck controls
 
-    This will load the methylprep control_probes.pkl file and perform all the calculations listed in
-    https://support.illumina.com/content/dam/illumina-support/documents/documentation/chemistry_documentation/infinium_assays/infinium_hd_methylation/beadarray-controls-reporter-user-guide-1000000004009-00.pdf
-    then produce a tidy color-coded XLSX file with results.
+    This will load the methylprep control_probes.pkl file and perform all the calculations and produce a tidy color-coded XLSX file with results.
 
     Note: this function is analogous to methylcheck.plot_controls() except that the output if a color-coded excel sheet instead of image charts.
 
-Required:
+Required Args:
     filepath (str or Path) to input files
 
 Optional Arguments:
@@ -864,10 +890,9 @@ Optional Arguments:
         You can increase this number to apply more stringent benchmarks for what constitutes good
         data, typically in the range of 1.0 to 3.0 (as a 1X to 3X cutoff multiplier).
     legacy (False)
-        True: changes XLSX column names to match BeadArray output file exactly.
-        So columns are TitleCase instead of readable text; drops the formulas row;
+        True: changes XLSX column names; columns are TitleCase instead of readable text; drops the formulas row;
         splits sample column into two SentrixID columns. Rounds all numbers to 1 decimal
-        instead of 2. Use this if you have automation that uses the Illumina legacy format.
+        instead of 2.
         This will also exclude p-value poobah column and the result (interpretation) column from report.
     roundoff (2, int)
         Option to adjust the level of rounding in report output numbers.
@@ -880,9 +905,16 @@ Optional Arguments:
         Note that this calculation also weights failures by how badly the miss the threshold value,
         so if any tests is a massive failure, the sample fails. Also, if pval(poobah) is included, and
         over 20% of probes fail, the sample fails regardless of other tests.
+    project_name (str, default None)
+        If defined, will be used to name the QC report file.
     """
+    if len(args) == 1:
+        kwargs['filepath'] = args[0]
+        LOGGER.warning(f"Assigned argument 0 `{kwargs['filepath']}` to `filepath` kwarg. This function takes 0 arguments.")
+    elif len(args) > 1:
+        raise AttributeError("This function takes 0 arguments, but {len(args)} arguments found. Specify the filepath using filepath=<path>.")
     if 'filepath' not in kwargs:
         raise FileNotFoundError("No control probes file location specified.")
-    reporter = BeadArrayControlsReporter(**kwargs)
+    reporter = ControlsReporter(**kwargs)
     reporter.process()
     reporter.save()
