@@ -21,7 +21,7 @@ this is analogous to using the methylcheck CLI, but for notebooks/scripts
 
 required:
     df: (required)
-        - data as a DataFrame of beta values
+        - data as a DataFrame of beta values (or DataFrame of m_values)
         - sample names in columns and probes in rows
 
 parameters:
@@ -219,9 +219,7 @@ def detection_poobah(poobah_df, pval_cutoff=0.05):
 class ReportPDF:
     """ReportPDF allows you to build custom QC reports.
 
--------
 To use:
--------
 
 - First, initialize the report and pass in kwargs, like ``myReport = ReportPDF(**kwargs)``
 - Next, run ```myReport.run_qc()`` to fill it in.
@@ -233,9 +231,7 @@ To use:
 - include 'path' with the path to your processed pickle files.
 - include an optional 'outpath' for where to save the pdf report.
 
--------
 kwargs:
--------
 
 - processing params
   - filename
@@ -260,45 +256,55 @@ kwargs:
   - M_vs_U (default False)
   - M_vs_U_compare (default False) -- shows the effect of all processing steps vs raw intensity
   - qc_signal_intensity
-  - controls (A batter of ported Genome Studio plots)
+  - controls (A battery of probe performance plots)
   - probe_types
+- customizing plots
+  - poobah_colormap (pass in the matplotlib colormap name to override the meta_mds default colormap)
+    This also overrides the default colormap used in M_vs_U plot.
+  - extend_poobah_range (Default: True will show 7 colors for poobah failure range on beta_mds_plot, max 30%; False will show only 5, max 20%)
+  - cutoff_line -- False to disable cutoff line on qc_signal_intensity and M_vs_U plots
+  - appendix_fontsize (default 12 point) -- specify an int for other fontsize
 
---------------
 custom tables:
---------------
 
 pass in arbitrary data using kwarg ``custom_tables`` as list of dictionaries with this structure:
 
 ```python
 custom_tables=[
-    {
-    'title': "some title, optional",
-    'col_names': ["<list of strings>"],
-    'row_names': ["<list of strings, optional>"],
-    'data': ["<list of lists, with order matching col_names>"],
-    'order_after': ["string name of the plot this should come after. It cannot appear first in list."]
-    },
-    {"<...second table here...>"}
+{
+'title': "some title, optional", # NOTE: chart titles must be unique!
+'col_names': ["<list of strings>"],
+'row_names': ["<list of strings, optional>"],
+'data': ["<list of lists, with order matching col_names>"],
+'order_after': "string name of the plot this should come after. It cannot appear first in list.",
+'font_size': <can be None, int, 'auto' (shrink to page), or 'truncate' (chop of long values to make fit)>
+},
+{"<...second table here...>"}
 ]
 ```
 
-------------------------
-Pre-processing pipeline:
-------------------------
+If 'order_after' is None, the custom table will be inserted at the beginning of the report.
+If there are multiple custom tables and all have 'order_after' set to None, the first table
+in the list gets inserted, then the next one, sequentially, so that the last table inserted
+will be the first table to appear.
 
-    Probe-level (w/explanations of suggested exclusions)
-        - Links to recommended probe exclusion lists/files/papers
-        - Background subtraction and normalization ('noob')
-        - Detection p-value ('neg' vs 'oob')
-        - Dye-bias correction (from SeSAMe)
-    Sample-level (w/explanations of suggested exclusions)
-        Detection p-value (% failed probes)
-        - custom detection (% failed, of those in a user-defined-list supplied to function)
-        MDS
-        Suggested for customer to do on their own
-        - Sex check
-        - Age check
-        - SNP check
+Pre-processing pipeline:
+
+Probe-level (w/explanations of suggested exclusions)
+    - Links to recommended probe exclusion lists/files/papers
+    - Background subtraction and normalization ('noob')
+    - Detection p-value ('neg' vs 'oob')
+    - Dye-bias correction (from SeSAMe)
+
+Sample-level (w/explanations of suggested exclusions)
+    Detection p-value (% failed probes)
+    - custom detection (% failed, of those in a user-defined-list supplied to function)
+    MDS
+
+Suggested for customer to do on their own
+    - Sex check
+    - Age check
+    - SNP check
     """
     # larger font
     # based on 16pt with 0.1 (10% of page) margins around it: use 80, 26, 16
@@ -356,20 +362,25 @@ Pre-processing pipeline:
             'gct_score': self.__dict__.get('gct',True), # part of detection_poobah table
         }
         self.plots = {
-            'beta_density_plot': self.__dict__.get('beta_density_plot',True),
-            'qc_signal_intensity': self.__dict__.get('qc_signal_intensity',True),
-            'M_vs_U_compare': self.__dict__.get('M_vs_U_compare',False),
-            'M_vs_U': self.__dict__.get('M_vs_U',False),
-            'predict_sex': self.__dict__.get('predict_sex',False),
-            'controls': self.__dict__.get('controls',True), # genome studio plots
-            'probe_types': self.__dict__.get('probe_types',True),
+            'beta_density_plot': self.__dict__.get('beta_density_plot', True),
+            'qc_signal_intensity': self.__dict__.get('qc_signal_intensity', True),
+            'M_vs_U_compare': self.__dict__.get('M_vs_U_compare', False),
+            'M_vs_U': self.__dict__.get('M_vs_U', False),
+            'predict_sex': self.__dict__.get('predict_sex', False),
+            'controls': self.__dict__.get('controls', True),
+            'probe_types': self.__dict__.get('probe_types', True),
         }
 
         self.custom = {}
         if 'custom_tables' in self.__dict__:
             self.parse_custom_tables(self.__dict__['custom_tables'])
             LOGGER.info(f"Found custom_tables and inserted into order: {self.order}")
-            LOGGER.info(self.custom)
+            #LOGGER.info(self.custom)
+
+        self.poobah_colormap = kwargs.get('poobah_colormap', None)
+        self.extend_poobah_range = kwargs.get('extend_poobah_range', True)
+        self.cutoff_line = kwargs.get('cutoff_line', True)
+        self.appendix_fontsize = kwargs.get('appendix_fontsize', None) # reverts to self.FONTSIZE if blank
 
         if self.__dict__.get('runme') == True:
             self.run_qc()
@@ -381,7 +392,7 @@ Pre-processing pipeline:
         'col_names': [list of strings],
         'row_names': [list of strings, optional],
         'data': [list of lists, with order matching col_names],
-        'order_after': [string name of the plot this should come after. It cannot appear first in list.]
+        'order_after': string name of the plot this should come after. It cannot appear first in list.
         }"""
         for table in tables:
             required_attributes = {'title', 'data', 'order_after', 'col_names'}
@@ -389,12 +400,16 @@ Pre-processing pipeline:
                 if i not in table:
                     raise KeyError("Your custom table must contain these keys: {required_attributes} (row_names is optional)")
             #1 place order -- refer to this later in self.custom
-            try:
-                _index = self.order.index(table['order_after'])
-                self.order.insert(_index, table['title'])
-                self.custom[table['title']] = table
-            except ValueError:
-                raise ValueError(f"Your custom table's 'order_after' label is not in this list of chart objects, so could not be ordered: {self.order}")
+            if table['order_after'] == None: # key must be present and set to None
+                _index = 0 # insert the table at start of report.
+            else:
+                try:
+                    _index = self.order.index(table['order_after']) + 1
+                except ValueError:
+                    raise ValueError(f"Your custom table's 'order_after' label is not in this list of chart objects, so could not be ordered: {self.order}")
+            self.order.insert(_index, table['title'])
+            self.custom[table['title']] = table
+
             if not isinstance(table['col_names'], list):
                 raise TypeError(f"Your custom table 'col_names' must be a list.")
             if table.get('row_names') and not isinstance(table['row_names'], list):
@@ -448,22 +463,12 @@ Pre-processing pipeline:
         except FileNotFoundError:
             raise FileNotFoundError("Could not load pickle files")
 
-        '''
-        self.functions = { # NOT USED --  needed to provide different conditions for each function below. #
-            'detection_poobah': detection_poobah,
-            'mds': methylcheck.beta_mds_plot,
-            'auto_qc': None,
-            'beta_density_plot': methylcheck.beta_density_plot,
-            'M_vs_U': methylcheck.plot_M_vs_U,
-            'controls': methylcheck.plot_controls,
-            'qc_signal_intensity': methylcheck.qc_signal_intensity,
-        }
-        '''
         if 'mds' in self.tests and len(beta_df.columns) > 1:
             # some things must be calculated ahead of time, because used twice
-            beta_mds_fig, ax, df_indexes_to_retain = methylcheck.beta_mds_plot(beta_df, silent=True, multi_params={'return_plot_obj':True, 'draw_box':True})
+            poobah_path = Path(path,'poobah_values.pkl') if 'detection_poobah' in self.order else None
+            beta_mds_fig, ax, df_indexes_to_retain = methylcheck.beta_mds_plot(beta_df, silent=True, multi_params={'return_plot_obj':True, 'draw_box':True},
+                poobah=poobah_path, palette=self.poobah_colormap, extend_poobah_range=self.extend_poobah_range)
             mds_passing = [sample_id for idx,sample_id in enumerate(beta_df.columns) if idx in df_indexes_to_retain]
-            print(mds_passing)
             include_mds = True
         else:
             beta_mds_fig = None
@@ -481,7 +486,7 @@ Pre-processing pipeline:
                         else:
                             sample_gct_percent_dict = {}
                         sample_poobah_failures = {sample_id: ("pass" if percent > min_allowed else "fail") for sample_id,percent in sample_percent_failed_probes_dict.items()}
-                        LOGGER.info(f"Poobah: {len([k for k in sample_poobah_failures.values() if k =='fail'])} failure(s) out of {len(sample_poobah_failures)} samples.")
+                        #LOGGER.info(f"Poobah: {len([k for k in sample_poobah_failures.values() if k =='fail'])} failure(s) out of {len(sample_poobah_failures)} samples.")
 
                         list_of_lists = []
                         col_names=['Sample_ID', 'Percent', 'Pass/Fail']
@@ -490,7 +495,7 @@ Pre-processing pipeline:
                         if include_mds:
                             col_names=['Sample_ID', 'MDS', 'Percent', 'Pass/Fail']
                         if self.tests['gct_score'] and include_mds:
-                            col_names=['Sample_ID', 'GCT score', 'MDS', 'Poobah (%)', 'Poobah Pass/Fail']
+                            col_names=['Sample_ID', 'GCT score', 'MDS', 'Passing Probes (%)', 'Pass/Fail']
 
                         for sample_id,percent in sample_percent_failed_probes_dict.items():
                             if self.tests['gct_score'] and sample_id in sample_gct_percent_dict:
@@ -535,7 +540,8 @@ Pre-processing pipeline:
                         self.plt.close()
                     elif part == 'M_vs_U':
                         LOGGER.info(f"M_vs_U plot")
-                        fig = methylcheck.plot_M_vs_U(meth=meth_df, unmeth=unmeth_df, noob=True, silent=True, verbose=False, plot=True, compare=False, return_fig=True, poobah=poobah_df)
+                        fig = methylcheck.plot_M_vs_U(meth=meth_df, unmeth=unmeth_df, noob=True, silent=False, verbose=True, plot=True,
+                            compare=False, return_fig=True, poobah=poobah_df, palette=self.poobah_colormap, cutoff_line=self.cutoff_line)
                         self.pdf.savefig(fig)
                         self.plt.close()
                         # if not plotting, it will return dict with meth median and unmeth median.
@@ -547,7 +553,7 @@ Pre-processing pipeline:
                         # if not plotting, it will return dict with meth median and unmeth median.
                     elif part == 'qc_signal_intensity':
                         LOGGER.info(f"QC signal intensity plot")
-                        fig = methylcheck.qc_signal_intensity(meth=meth_df, unmeth=unmeth_df, silent=True, return_fig=True, poobah=poobah_df)
+                        fig = methylcheck.qc_signal_intensity(meth=meth_df, unmeth=unmeth_df, silent=True, return_fig=True, poobah=poobah_df, cutoff_line=self.cutoff_line, palette=self.poobah_colormap)
                         self.pdf.savefig(fig)
                         self.plt.close()
                     elif part == 'controls':
@@ -581,7 +587,8 @@ Pre-processing pipeline:
             elif part in self.custom:
                 table = self.custom[part]
                 self.to_table(table['data'], col_names=table['col_names'],
-                    row_names=table.get('row_names'), add_title=table['title'])
+                    row_names=table.get('row_names'), add_title=table['title'],
+                    font_size=table.get('font_size',None))
 
 
         # and finally, appendix of log messages
@@ -610,7 +617,7 @@ Pre-processing pipeline:
             if rows_per_page <= 0:
                 LOGGER.error("couldn't write log messages to appendix")
                 break
-            self.page_of_paragraphs(para_list, self.pdf, line_height='single')
+            self.page_of_paragraphs(para_list, self.pdf, line_height='single', fontsize=self.appendix_fontsize)
             last_row += rows_per_page
         self.errors.close()
 
@@ -639,28 +646,31 @@ Pre-processing pipeline:
         exec_summary_samples = [] # temp storage lookup
         pass
 
-    def page_of_text(self, text, pdf):
+    def page_of_text(self, text, pdf, fontsize=None):
         """text is a single big string of text, with whitespace for line breaks.
         https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.text.html (0,0) is lower left; (1,1) is upper right """
-        print([len(i.split('\n')) for i in [text]])
+        #print([len(i.split('\n')) for i in [text]])
         firstPage = self.plt.figure(figsize=(11,8.5))
         firstPage.clf()
         wrapped_txt = self.textwrap.fill(text, width=self.MAXWIDTH)
         #wrapped_txt_list = textwrap.wrap(txt, width=80)
         # next, identify paragraph breaks here as strings of writespace, and wrap each part separately? then move onto page.
         #firstPage.text(0.5,0.5,txt, size=12, ha="left") # transform=firstPage.transFigure
-        firstPage.text(self.ORIGIN[0], self.ORIGIN[1], wrapped_txt, size=self.FONTSIZE)
+        fontsize = self.FONTSIZE if fontsize is None else fontsize
+        firstPage.text(self.ORIGIN[0], self.ORIGIN[1], wrapped_txt, size=fontsize)
         pdf.savefig()
         self.plt.close()
 
 
-    def page_of_paragraphs(self, para_list, pdf, line_height='double'):
+    def page_of_paragraphs(self, para_list, pdf, line_height='double', fontsize=None):
         """ https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.text.html (0,0) is lower left; (1,1) is upper right.
-        this version estimates the size of each paragraph and moves the origin downward accordingly.
-        tricky because the anchors are lower left, not upper left.
 
-        ok if a paragraph contains whitespace line breaks, OR each para is one long line to be wrapped here.
-        also - if a paragraph wraps, this accounts for it in total lines count"""
+        This version estimates the size of each paragraph and moves the origin downward accordingly.
+        Thsis is tricky because the anchors are lower left, not upper left.
+
+        It is ok if a paragraph contains whitespace line breaks, OR each paragraph is one long line to be wrapped here.
+        Also - if a paragraph wraps, this accounts for it in total lines count, so everything fits on a page.
+        The default fontsize is 12 if not specified."""
         #1 - finding the start points for each paragraph in page, counting backwards, so first paragraph is at top of page.
         para_lengths = [len(i.split('\n')) for i in para_list]
         extra_line_wraps = [max(len(self.textwrap.wrap(para, width=self.MAXWIDTH)) -1, 0) for para in para_list]
@@ -697,7 +707,8 @@ Pre-processing pipeline:
                 extra_lines.append(wrapped_txt)
                 #print(current_y_position, wrapped_addtl_line_count, wrapped_txt)
             else:
-                firstPage.text(x_margin, current_y_position, wrapped_txt, size=self.FONTSIZE)
+                fontsize = self.FONTSIZE if fontsize is None else fontsize
+                firstPage.text(x_margin, current_y_position, wrapped_txt, size=fontsize)
         if extra_lines != []:
             print("WARNING: DID NOT FIT")
             for extra_line in extra_lines:
@@ -706,11 +717,16 @@ Pre-processing pipeline:
         pdf.savefig()
         self.plt.close()
 
-    def to_table(self, list_of_lists, col_names, row_names=None, add_title=''):
+    def to_table(self, list_of_lists, col_names, row_names=None, add_title='', font_size='auto'):
         """
         - embeds a table in a PDF page.
         - attempts to split long tables into multiple pages.
-        - should warn if table is too wide to fit. """
+        - should warn if table is too wide to fit.
+        - font_size:
+          - auto: let matplotlib figure out the best size. This breaks if one column or value is soooooooper long.
+          - truncate: IN FUTURE, chop off each value that is too long, to force it to fit on a page. Data loss.
+          - INT: set the font for this table at this size (for control freaks)
+          - None: default -- use font size 12."""
 
         # stringify numbers; replace None
         for idx, sub in enumerate(list_of_lists):
@@ -720,10 +736,55 @@ Pre-processing pipeline:
         pages = math.ceil(len(list_of_lists)/rows_per_page)
         #paginate table
         for page in range(pages):
-            #LOGGER.debug(f'page {page+1} -- {(page * rows_per_page)}:{(page + 1) * rows_per_page}')
+            #LOGGER.debug(f'page, cols: {len(col_names)}, rows: {len(list_of_lists)}')
+            truncate = False
+            if isinstance(font_size, str) and font_size.lower() == 'truncate':
+                # overwrites list_of_lists so must happen before table is created
+                truncate = True
+                truncate_font_size = 12
+                try:
+                    max_field_length = 24
+                    # default max length = 24 chars; if more than 4 columns, reduce by ((N-4)/20)*80 chars
+                    if 4 < len(col_names) < 10:
+                        max_field_length = int((24 - ((len(col_names)-4)/20) * 80))
+                        if max_field_length <= 4: # 9 col
+                            truncate_font_size = 6
+                            max_field_length += 8
+                        elif max_field_length <= 8: # 8 col
+                            truncate_font_size = 7
+                            max_field_length += 3
+                        elif max_field_length <= 12: # 7 col
+                            truncate_font_size = 7
+                            max_field_length += 1
+                        elif max_field_length <= 16: # 6 col
+                            truncate_font_size = 7
+                            #max_field_length += 1
+                        elif max_field_length <= 20: # 5 col
+                            truncate_font_size = 7
+                            #max_field_length += 0
+                        elif max_field_length <= 24: # 4 col
+                            truncate_font_size = 8
+                            #max_field_length += 0
+                    elif len(col_names) >= 10:
+                        LOGGER.error(f"Could not truncate long fields in custom table because there were 10 or more columns.")
+                        truncate = False
+                    if truncate is True:
+                        for row_num, row in enumerate(list_of_lists.copy()):
+                            for field_num,field in enumerate(row):
+                                if len(str(field)) > max_field_length:
+                                    list_of_lists[row_num][field_num] = str(field)[:max_field_length] + '..'
+                                    if self.debug:
+                                        LOGGER.info(f"{field} truncated to {field[:max_field_length]}..")
+                        for field_num,field in enumerate(col_names):
+                            if len(str(field)) > max_field_length:
+                                col_names[field_num] = str(field)[:max_field_length] + '..'
+                                if self.debug:
+                                    LOGGER.info(f"column name {field} truncated to {field[:max_field_length]}..")
+                except Exception as e:
+                    truncate = False
+                    LOGGER.error(f"Could not truncate long fields in custom table: {e}")
             page_data = list_of_lists[(page * rows_per_page) : (page + 1) * rows_per_page]
-            fig, ax = self.plt.subplots(1, figsize=(10,8))
-            #ax.axis('tight')
+            fig, ax = self.plt.subplots(1, figsize=(8,10.5))
             ax.axis('off')
             matplotlib_table = ax.table(
                 cellText=page_data,
@@ -732,9 +793,26 @@ Pre-processing pipeline:
                 edges='open',
                 colLoc='right',
                 )
-            matplotlib_table.auto_set_font_size(False)
-            matplotlib_table.set_fontsize(12)
+
+            if font_size is None:
+                matplotlib_table.auto_set_font_size(False)
+                matplotlib_table.set_fontsize(12)
+            elif isinstance(font_size, int):
+                matplotlib_table.auto_set_font_size(False)
+                matplotlib_table.set_fontsize(font_size)
+            elif ( (isinstance(font_size, str) and font_size.lower() == 'auto') or
+                   (isinstance(font_size, str) and font_size.lower() == 'truncate' and truncate is False) ): #default shrinks to fit page
+                matplotlib_table.auto_set_font_size(True)
+                scale_ratio = 1.2 # use smaller page margins
+                matplotlib_table.scale(scale_ratio, scale_ratio)
+            elif truncate is True:
+                matplotlib_table.auto_set_column_width(col=list(range(len(col_names)))) # Provide integer list of columns to adjust
+                matplotlib_table.auto_set_font_size(False)
+                matplotlib_table.set_fontsize(truncate_font_size)
+                scale_ratio = 1.2
+                matplotlib_table.scale(scale_ratio, scale_ratio)
             if page == 0:
                 self.plt.title(add_title, y=1.1) #pad=20) # -- placement is off
+            #fig.set_size_inches(8,10.5)
             self.pdf.savefig(fig)
             self.plt.close()
