@@ -3,6 +3,7 @@
 import logging
 import os
 import datetime
+from itertools import cycle
 
 # package
 import pandas as pd
@@ -10,6 +11,7 @@ import numpy as np
 import seaborn as sns
 import matplotlib.pyplot as plt
 import matplotlib.lines as mlines
+from matplotlib.cm import get_cmap
 from pathlib import Path
 
 # because sklearn is a HUGE library and we're only using a single function from it,
@@ -72,7 +74,7 @@ def mean_beta_plot(df, verbose=False, save=False, silent=False):
 
 def beta_density_plot(df, verbose=False, save=False, silent=False, reduce=0.1, plot_title=None, ymax=None, return_fig=False, full_range=False, highlight_samples=None, figsize=(12,9), show_labels=None, filename='beta.png'):
     """Returns a plot of beta values for each sample in a batch of samples as a separate line.
-    Y-axis values is the count (of what? intensity? normalized?).
+    Y-axis values is an arbitrary scale, similar to a histogram of probes that have a given beta value.
     X-axis values are beta values (0 to 1) for a single samples
 
     Input (df):
@@ -239,8 +241,9 @@ def sample_plot(df, **kwargs):
 
 
 def cumulative_sum_beta_distribution(df, cutoff=0.7, verbose=False, save=False, silent=False):
-    """ attempts to filter outlier samples based on the cumulative area under the curve
-    exceeding a reasonable value (cutoff).
+    """Attempts to filter outlier samples based on the cumulative area under the curve
+    exceeding a reasonable value (cutoff). This method only works on poor quality samples that are
+    better identified using ControlReporter summary (CLI: 'methylcheck controls')
 
     Inputs:
         DataFrame -- wide format (probes in columns, samples in rows)
@@ -294,11 +297,11 @@ def cumulative_sum_beta_distribution(df, cutoff=0.7, verbose=False, save=False,
     return df.drop(outliers, axis=0)
 
 
-def beta_mds_plot(df, filter_stdev=1.5, verbose=False, save=False, silent=False, multi_params={'draw_box':True}, plot_removed=False, nafill='quick', poobah=None, palette="spectral"):
+def beta_mds_plot(df, filter_stdev=1.5, verbose=False, save=False, silent=False, multi_params={'draw_box':True}, plot_removed=False,
+    nafill='quick', poobah=None, palette=None, labels=None, extend_poobah_range=True, plot=True):
     """Performs multidimensional scaling on a dataframe of samples
 
 Arguments
----------
 
     ``df``:
         dataframe of beta values for a batch of samples (rows are probes; cols are samples)
@@ -307,6 +310,25 @@ Arguments
         the fraction of samples to include, based on the standard deviation of this batch of samples.
         So using the default value of 1.5 means that all samples whose MDS-transformed beta sort_values
         are within +/- 1.5 standard deviations of the average beta are retained in the data returned.
+    ``plot_removed``:
+        if True, displays a plot of samples' beta-distributions that were removed by MDS filtering.
+        ignored if silent=True.
+    ``nafill``: ('quick' | 'impute')
+        by default, most samples will contain missing values where probes failed the signal-noise detection
+        in methylprep. By default, it will use the fastest method of filling in samples from adjacent sample's probe values
+        with the 'quick' method. Or, if you want it to use the average value for all samples for each probe, use 'impute', which will be much slower.
+    ``poobah``:
+        path to poobah_values.pkl file. Default is None. If supplied, this will color code dots according to
+        percent of failed probes for each sample as a second dimension of QC on the plot. Does not filter or affect the output dataframe returned.
+    ``palette``:
+        Optional - Specify a matplotlib/seaborn palette name, such as 'CMRmap_r', 'coolwarm', or 'nipy_spectral'.
+        Default is 'twilight'.
+    ``labels``:
+        pass in a dictionary with sample names found in ``df`` columns and a (number or string) representing the groups to assign samples to.
+        Use this to color-code the samples against a known classification scheme, such as cell type, and observe whether the MDS clustering
+        pattern aligns with this input parameter. This feature is not compatible with ``poobah`` or ``multi_params``.
+    ``extend_poobah_range``:
+        True means 7 colors appear covering 0-30%. False means 5 colors and 0-20%. Default is True.
     ``multi_params``:
         is a dict, passed into this function from a multi-compare-MDS wrapper function, containing:
         {return_plot_obj=True,
@@ -316,18 +338,8 @@ Arguments
         xy_lim=None,
         color_num=0,
         PSF=1.2 -- plot scale factor (margin beyond points to display)}
-    ``plot_removed``:
-        if True, displays a plot of samples' beta-distributions that were removed by MDS filtering.
-        ignored if silent=True.
-    ``nafill``: ('quick' | 'impute')
-        by default, most samples will contain missing values where probes failed the signal-noise detection
-        in methylprep. By default, it will use the fastest method of filling in samples from adjacent sample's probe values
-        with the 'quick' method. Or, if you want it to use the average value for all samples for each probe, use 'impute', which will be much slower.
-    ``poobah``: path to poobah_values.pkl file. Default is None. If supplied, this will color code dots according to
-        percent of failed probes for each sample as a second dimension of QC on the plot. Does not filter or affect the output dataframe returned.
 
 Options
---------
 
     ``verbose``:
         If True, provides additional messages
@@ -335,20 +347,19 @@ Options
     ``silent``:
         - if running from command line in an automated process, you can run in `silent` mode to suppress any user interaction.
         - In this case, whatever `filter_stdev` you assign is the final value, and a file will be processed with that param.
-        - Silent also suppresses plots (images) from being generated. only files are returned.
+    ``plot``: (default True)
+        - plot is False, this suppresses plots (images) from being generated and shown on screen.
+        - .png files are still saved if ``save`` == True.
 
-returns
--------
+Returns
 
     Returns a filtered dataframe. If ``return_plot_obj`` is True, it returns the plot, for making overlays in ``methylize``.
 
-requires
---------
+Requires
 
     pandas, numpy, pyplot, sklearn.manifold.MDS
 
-notes
------
+Notes
 
     this will remove probes from ALL samples in batch from consideration if any samples contain NaN (missing values) for that probe.
 
@@ -514,6 +525,7 @@ notes
                                    'xkcd:olive', 'xkcd:lavender', 'xkcd:indigo', 'xkcd:black', 'xkcd:azure', 'xkcd:brown', 'xkcd:aquamarine', 'xkcd:darkblue']))
 
         # these are used with poobah as a scale of failure rate
+        """
         poobah_palettes = {
         "magma": dict(enumerate(
             [(0.15, 0.6, 0.4), (0.3, 0.25, 0.60), (0.44, 0.25, 0.5), (0.7, 0.2, 0.2), (0.973, 0.462, 0.36), (0.68, 0.60, 0.25), (0.9, 0.8, 0.2)]
@@ -525,7 +537,28 @@ notes
             [(0.275191, 0.194905, 0.496005), (0.212395, 0.359683, 0.55171), (0.153364, 0.497, 0.557724), (0.122312, 0.633153, 0.530398), (0.288921, 0.758394, 0.428426), (0.626579, 0.854645, 0.223353)]
         ))
         }
-        sb_palette = poobah_palettes.get(palette, poobah_palettes.get("magma"))
+        # good options: gist_earth_r, CMRmap_r, coolwarm, hot_r, nipy_spectral
+
+        if palette is not None and palette not in poobah_palettes:
+            try:
+                sb_palette = {}
+                linear_segmented_cmap = get_cmap(palette)
+                N_colors = 7 if extend_poobah_range else 5
+                for i in range(N_colors):
+                    sb_palette[i] = linear_segmented_cmap((i+1)/float(N_colors+1)) # adding 1 to N fixes colors, because zero-end can be all white in some palettes
+            except ValueError:
+                LOGGER.warning(f"{palette} not a valid seaborn/matplotlib colormap name, defaulting to 'twilight'.")
+                sb_palette = poobah_palettes.get("twilight")
+        else:
+            sb_palette = poobah_palettes.get(palette, poobah_palettes.get("twilight"))
+        """
+        sb_palette = dict(enumerate(sns.color_palette("twilight", n_colors=7 if extend_poobah_range else 5, desat=0.8)))
+        if palette:
+            try:
+                sb_palette = dict(enumerate(sns.color_palette(palette, n_colors=7 if extend_poobah_range else 5, desat=0.8)))
+            except ValueError:
+                LOGGER.warning(f"{palette} not a valid seaborn/matplotlib colormap name, defaulting to 'twilight'.")
+
 
         if multi_params.get('fig') == None:
             fig = plt.figure(figsize=(12, 9))
@@ -545,7 +578,7 @@ notes
             ax = multi_params.get('ax')
 
         if isinstance(poobah, pd.DataFrame):
-            percent_failures_hues, legend_order = _add_poobah(poobah)
+            percent_failures_hues, legend_order = _add_poobah(poobah, extended=extend_poobah_range)
             poobah_color_lookup = list(enumerate(percent_failures_hues)) # list of tuples with (0, '5 to 10') data. numbers should match md2
             color_lookup = {legend_group:[] for legend_group in legend_order}
             for N,legend_group in poobah_color_lookup:
@@ -556,13 +589,36 @@ notes
             for legend_group in legend_order:
                 this_x = mds_transformed[color_lookup[legend_group], 0]
                 this_y = mds_transformed[color_lookup[legend_group], 1]
-                ax.scatter(this_x, this_y, s=DOTSIZE, color=sb_palette.get(color_num,'black'), label=legend_group)
+                ax.scatter(this_x, this_y, s=DOTSIZE, color=sb_palette.get(color_num, 'black'), label=legend_group)
                 color_num += 1
-            ax.legend(title="p-value failure (%)")
+            ax.legend(title="Probe failure rate (%)")
         else:
             ax.scatter(mds_transformed[:, 0], mds_transformed[:, 1], s=DOTSIZE, color='xkcd:ivory', edgecolor='black', linewidth=0.4) # EXCLUDED
             ax.scatter(md2[:, 0], md2[:, 1], s=DOTSIZE, color=COLORSET.get(color_num,'black'), edgecolor='black', linewidth=0.4) # RETAINED
 
+        if isinstance(poobah, pd.DataFrame) and isinstance(labels, dict):
+            print("ERORR: You cannot apply both poobah and labels; labels will be ignored.")
+        elif isinstance(labels, dict): # TODO: support pd.Series here
+            # THIS WILL WRITE OVER the black/grey dots already on plot.
+            label_color_lookup = list(enumerate(labels.values()))
+            legend_order = list(set([v for k,v in labels.items()])) # in order seen
+            color_lookup = {legend_group:[] for legend_group in legend_order}
+            for N,legend_group in label_color_lookup:
+                color_lookup[legend_group].append(N)
+            ax.scatter(mds_transformed[:, 0], mds_transformed[:, 1], s=DOTSIZE, c='xkcd:ivory', edgecolor='black', linewidth=0.4)
+            ax.scatter(md2[:, 0], md2[:, 1], s=DOTSIZE, c=COLORSET.get(color_num,'black'))
+            color_num = 0
+            for legend_group in legend_order:
+                try:
+                    this_x = mds_transformed[color_lookup[legend_group], 0]
+                    this_y = mds_transformed[color_lookup[legend_group], 1]
+                    ax.scatter(this_x, this_y, s=DOTSIZE, color=sb_palette.get(color_num,'black'), label=legend_group)
+                    color_num += 1
+                except IndexError as e:
+                    # if the dict passed in doesn't map perfectly to the data, you'll see this error.
+                    LOGGER.error(f"Failed to assign legend group {legend_group} to plot: {e}; Double check that your labels dict matches the shape of your data exactly.")
+            ax.legend(title="Sample Categories")
+
         x_range_min = PSF*old_X_range[0] if PSF*old_X_range[0] < minX else PSF*minX
         x_range_max = PSF*old_X_range[1] if PSF*old_X_range[1] > maxX else PSF*maxX
         y_range_min = PSF*old_Y_range[0] if PSF*old_Y_range[0] < minY else PSF*minY
@@ -580,7 +636,11 @@ notes
 
         if save:
             saved_fig = plt.gcf()
-        if silent == True:
+        if plot:
+            plt.show()
+            plt.close(fig)
+        """
+        else:
             # take the original dataframe (df) and remove samples that are outside the sample thresholds, returning a new dataframe
             df.drop(df.index[df_indexes_to_exclude], inplace=True)
             image_name = df.index.name or f'beta_mds_n={len(df.index)}'
@@ -591,21 +651,21 @@ notes
             plt.close(fig)
             # returning DataFrame in original structure: rows are probes; cols are samples.
             return df  # may need to transpose this first.
-        else:
-            plt.show()
-            plt.close(fig)
-
+        """
         ########## BEGIN INTERACTIVE MODE #############
-        print(f"{mds_transformed.shape[0]} original samples; {md2.shape[0]} after filtering")
-        print('Your scale factor was: {0}'.format(adj))
-        adj = input("Enter new scale factor, <enter> to accept and save: ")
-        if adj == '':
+        if not silent:
+            print(f"{mds_transformed.shape[0]} original samples; {md2.shape[0]} after filtering")
+            print('Your scale factor was: {0}'.format(adj))
+            adj = input("Enter new scale factor, <enter> to accept and save: ")
+            if adj == '':
+                break
+            try:
+                adj = float(adj)
+            except ValueError:
+                print("Not a valid number. Type a number with a decimal value, or Press <enter> to quit.")
+                continue
+        else:
             break
-        try:
-            adj = float(adj)
-        except ValueError:
-            print("Not a valid number. Type a number with a decimal value, or Press <enter> to quit.")
-            continue
 
     prev_df = len(df)
     # take the original dataframe (df) and remove samples that are outside the sample thresholds, returning a new dataframe
@@ -649,6 +709,7 @@ notes
         plt.close(fig) # avoids displaying plot again in jupyter.
         if verbose:
             LOGGER.info("Saved {0}".format(outfile))
+
     # return DataFrame in original "long" format: rows are probes; cols are samples.
     df_out = original_df.drop(original_df.index[df_indexes_to_exclude])
     if df_out.shape[0] < df_out.shape[1]:
@@ -656,10 +717,13 @@ notes
     return df_out
 
 
-def _add_poobah(poobah):
-    """ used by beta_MDS_plot """
-    if poobah.isna().sum().sum() > 0:
-        LOGGER.warning("Your poobah_values.pkl file contains missing values; color coding may be inaccurate.")
+def _add_poobah(poobah, extended=True):
+    """ used by beta_MDS_plot to categorize poobah failure ranges for colormap.
+    - Default returns 7 groups (0-30%). But 5 groups (0-20%) also an option with 'extended=False'.
+    - Returns: a df with sample names in index and failure % in a column. """
+    #if poobah.isna().sum().sum() > 0:
+    #    LOGGER.warning("Your poobah_values.pkl file contains missing values; color coding may be inaccurate.")
+    # this happens normally with qualityMask True
     percent_failures = round(100*( poobah[poobah > 0.05].count() / poobah.count() ),1)
     percent_failures = percent_failures.rename('probe_failure_(%)')
     # Series.where will replace the stuff that is False, so you have to negate it.
@@ -667,16 +731,19 @@ def _add_poobah(poobah):
     percent_failures_hues.where(~percent_failures_hues.between(5,10), 1, inplace=True)
     percent_failures_hues.where(~percent_failures_hues.between(10,15), 2, inplace=True)
     percent_failures_hues.where(~percent_failures_hues.between(15,20), 3, inplace=True)
-    percent_failures_hues.where(~percent_failures_hues.between(20,25), 4, inplace=True)
-    percent_failures_hues.where(~percent_failures_hues.between(25,30), 5, inplace=True)
-    percent_failures_hues.where(~(percent_failures_hues > 30), 6, inplace=True)
-    percent_failures_hues = percent_failures_hues.astype(int)
-    percent_failures_hues = percent_failures_hues.replace({0:'0 to 5', 1:'5 to 10', 2:'10 to 15', 3:'15 to 20', 4:'20 to 25', 5:'25 to 30', 6:'>30'})
-    legend_order = ['0 to 5','5 to 10','10 to 15','15 to 20','20 to 25','25 to 30','>30']
-    #hues_palette = sb.color_palette("twilight", n_colors=7, desat=0.8) if palette is None else sb.color_palette(palette, n_colors=7, desat=0.8)
-    #this = sb.scatterplot(data=qc, x="mMed", y="uMed", hue="probe_failure_(%)",
-    #    palette=hues_palette, hue_order=legend_order, legend="full") # size="size"
-    return percent_failures_hues, legend_order # a df with sample names in index and failure % in a column.
+    if extended:
+        percent_failures_hues.where(~percent_failures_hues.between(20,25), 4, inplace=True)
+        percent_failures_hues.where(~percent_failures_hues.between(25,30), 5, inplace=True)
+        percent_failures_hues.where(~(percent_failures_hues > 30), 6, inplace=True)
+        percent_failures_hues = percent_failures_hues.astype(int)
+        percent_failures_hues = percent_failures_hues.replace({0:'0 to 5', 1:'5 to 10', 2:'10 to 15', 3:'15 to 20', 4:'20 to 25', 5:'25 to 30', 6:'>30'})
+        legend_order = ['0 to 5','5 to 10','10 to 15','15 to 20','20 to 25','25 to 30','>30']
+    else:
+        percent_failures_hues.where(~(percent_failures_hues > 20), 4, inplace=True)
+        percent_failures_hues = percent_failures_hues.astype(int)
+        percent_failures_hues = percent_failures_hues.replace({0:'0 to 5', 1:'5 to 10', 2:'10 to 15', 3:'15 to 20', 4:'>20'})
+        legend_order = ['0 to 5','5 to 10','10 to 15','15 to 20','>20']
+    return percent_failures_hues, legend_order
 
 
 def mean_beta_compare(df1, df2, save=False, verbose=False, silent=False):
@@ -907,8 +974,12 @@ returns:
     retained_sample_dfs = []
     for i in range(0, 4*len(list_of_dfs), 1):
         DD = fig.axes[0].collections[i]
-        DD.set_offset_position('data')
-        #print(DD.get_offsets())
+        try:
+            DD.set_offset_position('data') # this was removed in matplotlib 3.5+ and replaced with 'screen'
+            # according to docs, but 'screen' attribute here
+            #print(DD.get_offsets())
+        except:
+            pass
         all_coords.extend(DD.get_offsets().tolist())
         #print(i, len(all_coords))
         if i % 4 == 0: # 0, 4, 8 -- this is the first data set applied to plot. (x,y plot coords)
